@@ -1,23 +1,21 @@
-import matplotlib.pyplot as plt
 import numpy as np
+from scipy.signal import stft
+import matplotlib.pyplot as plt
+
 import plotly.graph_objects as go
 from plotly.colors import DEFAULT_PLOTLY_COLORS
 from ipywidgets import widgets, ValueWidget
-from pynwb.ecephys import LFP, SpikeEventSeries, ElectricalSeries
-from scipy.signal import stft
-import pynwb
 
-from .base import fig2widget, nwb2widget, lazy_tabs, render_dataframe
+from pynwb import TimeSeries
+from pynwb.base import DynamicTable
+from pynwb.ecephys import SpikeEventSeries, ElectricalSeries
+
+from .base import fig2widget, lazy_tabs, render_dataframe
 from .timeseries import BaseGroupedTraceWidget
 from .brains import HumanElectrodesPlotlyWidget
 
 
-def show_lfp(ndobj: LFP, neurodata_vis_spec: dict):
-    lfp = list(ndobj.electrical_series.values())[0]
-    return nwb2widget(lfp, neurodata_vis_spec)
-
-
-def show_spectrogram(nwbobj: pynwb.TimeSeries, channel=0, **kwargs):
+def show_spectrogram(nwbobj: TimeSeries, channel=0, **kwargs):
     fig, ax = plt.subplots()
     f, t, Zxx = stft(nwbobj.data[:, channel], nwbobj.rate, nperseg=2 * 17)
     ax.imshow(
@@ -33,13 +31,13 @@ def show_spectrogram(nwbobj: pynwb.TimeSeries, channel=0, **kwargs):
 
 
 class ElectrodeGroupsWidget(ValueWidget, widgets.HBox):
-    def __init__(self, nwbobj: pynwb.base.DynamicTable, **kwargs):
+    def __init__(self, nwbobj: DynamicTable, **kwargs):
         super().__init__()
         group_names = nwbobj.group_name[:]
         ugroups, group_pos, counts = np.unique(
             group_names, return_inverse=True, return_counts=True
         )
-        elec_pos = np.hstack(np.arange(count) for count in counts)
+        elec_pos = np.hstack(np.arange(count).tolist() for count in counts)
 
         hovertext = []
         df = nwbobj.to_dataframe()
@@ -79,7 +77,6 @@ class ElectrodeGroupsWidget(ValueWidget, widgets.HBox):
 
         def selection_fn(trace, points, selector):
             self.value = points.point_inds
-            print(self.value)
 
         self.fig.data[0].on_selection(selection_fn)
 
