@@ -1,26 +1,101 @@
-from pynwb.core import DynamicTable
+import numbers
+from typing import Iterable
+
 import numpy as np
+import pandas as pd
+
+from pynwb.core import DynamicTable
 
 
-def infer_categorical_columns(dynamic_table: DynamicTable):
+def extract_data_from_intervals(dynamic_table: DynamicTable):
+    # solution: https://stackoverflow.com/a/50297200/11483674
+    # data = [x if x == x else 'NaN' for x in dynamic_table[:]]
+    data = [x for x in dynamic_table[:] if x == x]
+    classes = pd.unique(data).tolist()
+    try:
+        data = [float(i) for i in data]
+        classes = [float(i) for i in classes]
+        classes.sort(key=float)
+    except:
+        pass
+    return data, classes
 
+
+def infer_categorical_columns(dynamic_table: DynamicTable, region: Iterable = None):
+    """
+    Parameters
+    ----------
+    dynamic_table: DynamicTable
+    region: Iterable
+        row indices to select
+
+    Returns
+    -------
+    categorical_cols: dict()
+        keys: as columns that are categorical, values as the unique values
+    """
     categorical_cols = {}
+    region = (
+        np.array(region, dtype="uint")
+        if region is not None
+        else np.arange(len(dynamic_table), dtype="uint")
+    )
     for name in dynamic_table.colnames:
         if len(dynamic_table[name].shape) == 1:
-            try:  # TODO: fix this
-                unique_vals = np.unique(dynamic_table[name].data)
-                if 1 < len(unique_vals) <= (len(dynamic_table[name].data) / 2):
+            try:
+                if isinstance(
+                    dynamic_table[name].data[0], (str, numbers.Number, bytes)
+                ):
+                    column_data = [dynamic_table[name].data[i] for i in region]
+                elif hasattr(dynamic_table[name].data[0], "name"):
+                    column_data = [dynamic_table[name].data[i].name for i in region]
+                else:
+                    continue
+                unique_vals = np.unique(column_data)
+                if not isinstance(unique_vals[0], str) and all(np.isnan(unique_vals)):
+                    continue
+                if 1 < len(unique_vals) <= (len(column_data) / 2):
                     unique_vals = [
                         x.decode() if isinstance(x, bytes) else x for x in unique_vals
                     ]  # handle h5py 3.0
-                    categorical_cols[name] = unique_vals
-            except:
-                pass
+                    categorical_cols[name] = np.array(column_data)
+            except Exception as e:
+                print(e)
     return categorical_cols
 
 
+def infer_columns_to_plot(dynamic_table: DynamicTable):
+    """Infer which columns can be plotted in summary widgets
+
+    Parameters
+    ----------
+    dynamic_table : DynamicTable
+
+    Returns
+    -------
+    column_names_to_plot: list
+        Columns that can be plotted with the dynamic table summary
+    """
+
+    column_names_to_plot = []
+
+    df = dynamic_table.to_dataframe()
+    categorical_columns = []
+    for name in dynamic_table.colnames:
+        # if name not in categorical_cols.keys():  # categorical columns can always be plotted
+        value = df[name].values[0]
+
+        if isinstance(value, (int, float, np.integer)):
+            column_names_to_plot.append(name)
+        elif isinstance(value, (str, bool, bytes)):
+            column_names_to_plot.append(name)
+            categorical_columns.append(name)
+
+    return column_names_to_plot, categorical_columns
+
+
 def group_and_sort(
-    group_vals=None, group_select=None, order_vals=None, discard_rows=None, limit=None
+    group_vals=None, group_select=None, order_vals=None, keep_rows=None, limit=None
 ):
     """
     Logical flow:
@@ -60,11 +135,10 @@ def group_and_sort(
             order = np.argsort(order_vals)
 
     # apply discard rows
-    if discard_rows is not None:
-        keep = np.logical_not(np.isin(order, discard_rows))
-        order = order[keep]
+    if keep_rows is not None:
+        order = order[keep_rows]
         if group_inds is not None:
-            group_inds = group_inds[keep]
+            group_inds = group_inds[keep_rows]
 
     # apply discard NaN categories
     try:
