@@ -1,7 +1,25 @@
-from ipywidgets import Output
-from ipywidgets.widgets.interaction import show_inline_matplotlib_plots, clear_output
 import asyncio
 
+import matplotlib.pyplot as plt
+
+from ipywidgets import Output
+from ipywidgets.widgets.interaction import clear_output
+import plotly.graph_objects as go
+
+
+def clean_axes(axes):
+    """
+    Removes top and right spines from axes
+
+    Parameters
+    ----------
+    axes: iterable
+
+    """
+    for ax in axes:
+        ax.spines["top"].set_visible(False)
+        ax.spines["right"].set_visible(False)
+
 
 def unpack_controls(controls, process_controls=lambda x: x):
     control_states = {}
@@ -15,7 +33,7 @@ def unpack_controls(controls, process_controls=lambda x: x):
     return kwargs
 
 
-def interactive_output(f, controls, process_controls=lambda x: x):
+def interactive_output(f, controls, process_controls=lambda x: x, fixed=None):
     """Connect widget controls to a function.
 
     This function does not generate a user interface for the widgets (unlike `interact`).
@@ -23,21 +41,30 @@ def interactive_output(f, controls, process_controls=lambda x: x):
     The user interface layout must be defined and displayed manually.
     """
 
+    if fixed is None:
+        fixed = dict()
+
     out = Output()
 
     def observer(change):
-        show_inline_matplotlib_plots()
         with out:
             clear_output(wait=True)
-            f(**unpack_controls(controls, process_controls))
-            show_inline_matplotlib_plots()
+            plot = f(**fixed, **unpack_controls(controls, process_controls))
+            plt.show()
 
     for k, w in controls.items():
         w.observe(observer, "value")
-    show_inline_matplotlib_plots()
     observer(None)
     return out
 
+def set_plotly_callbacks(f, controls, process_controls=lambda x: x):
+    fig = go.FigureWidget()
+
+    def observer(change):
+        return f(fig=fig, **unpack_controls(controls, process_controls))
+    for k, w in controls.items():
+        w.observe(observer, "value")
+    return observer(None)
 
 class Timer:
     def __init__(self, timeout, callback):
